function [ errorDeg, errorMM, calErr ] = crsLiveTrackCalibrateDevice(whichEye, pupil, glint, targets, viewDist)
% CRSLIVETRACKCALIBRATEDEVICE This function calibrates the device for one
% designated eye using the crsLiveTrack MATLAB Toolbox Library.
% ----------------------------------------------------------------------------
%
% crsLiveTrackCalibrateDevice
% =====================
%
% MATLAB Syntax
% =============
% [ calErr ] = crsLiveTrackCalibrateDevice(whichEye, pupil, glint, targets, viewDist)
%
% whichEye - 'left' to calibrate left eye or 'right' to calibrate right eye
%
% pupil - a matrix with x and y positions of the left eye pupil for each 
%         target position (2 columns and one row for each target position) 
%
% glint - a matrix with x and y positions of the left eye glint for each 
%         target position (2 columns and one row for each target position) 
%
% targets - a matrix with x and y positions of the target (2 columns and
%         one row for each target position) 
%
% viewDist - the viewing distance. The distance between the eyes and the
%         monitor in the same units as the "targets" e.g. millimetres
%
% Return values
% =============
% calErr(1) - An integer scalar where 0 represents a successful calibration
%
% calErr(2) - RMS calibration displacement error between target and calculated 
%             viewpoint positions (in millimetres)                      
% 
% History:
% 09/2017   MAH Created  

%% Access the Toolbox Constants and Calibrate Device

if strcmpi(whichEye,'leftEye') || strcmpi(whichEye,'left')
    eye=0;
    disp('Please wait while left eye is being calibrated...');
elseif strcmpi(whichEye,'rightEye') || strcmpi(whichEye,'right')
    eye=1;
    disp('Please wait while right eye is being calibrated...');
else
    error('ERROR: whichEye parameter must be either ''left'' or ''right''')
end

% Access Toolbox Constants
global CRS;

% create the calibration target structure matrix
a = nan(4,size(targets, 1));
for i = 1 : size(targets, 1)
    a(1, i) = targets(i, 1);
    a(2, i) = targets(i, 2);
    a(3, i) = pupil(i, 1) - glint(i, 1);
    a(4, i) = pupil(i, 2) - glint(i, 2);
end

% median glint of valid targets
glintMedian = median(glint(~any(isnan(a)),:));

% remove any un-tracked targets
a = a(:,~any(isnan(a)));

noOfFixations = size(a,2);

% check if at least five vlis fixations are left
if noOfFixations<5
    disp(['ERROR: Could not calibrate. At least five valid fixations are needed. ',...
        'Only ',num2str(size(a,2)),' valid fixations was found']);
    errorDeg = 0;
    errorMM = 0;
    calErr = 0;
    return
end

% Calibrate Device
calErr = crsLiveTrack(CRS.mexCalibrateDevice, eye, noOfFixations, a, viewDist, glintMedian);

if (calErr(1) ~= 0)
	disp('ERROR: Failed to calibrate LiveTrack device!');
    errorDeg = 0;
    errorMM = 0;
    calErr = 0;
    return
elseif (calErr(2) == 0) % calibration error cannot be zero
	disp('ERROR: Failed to calibrate LiveTrack device!');
    errorDeg = 0;
    errorMM = 0;
    calErr = 0;
    return
else
    if eye
        disp('LiveTrack successfully calibrated right eye.');
    else
        disp('LiveTrack successfully calibrated left eye.');
    end
end

% Calculate average error in millimetre
errorMM = sqrt(calErr(2)/noOfFixations); 

% Calculate average error in degrees of visual angle (only accurate for
% small angles)
errorDeg = atand(errorMM/viewDist); 

disp(['Average calibration error is: ',num2str(errorMM),' mm or ',num2str(errorDeg),' degrees.'])


