function crsLiveTrackDemoGazeContingentAperture(WindowID)
%CRSLIVETRACKDEMOGAZECONTINGENTAPERTURE  This demo shows how to get eye
%   tracking data from CRS LiveTrack eye trackers and plot a blurred spot
%   at the current gaze postion on the screen
%
%   The demo also shows how to make a quick drift/offset correction by
%   having the particiopant fixate on a single dot on the screen for one
%   second. This is done by pressing the "1" key.
%
%   Images can be changed by pressing the "2" key.
%
%   The input to this function must be:
%   WindowID - the ID of the stimulus monitor as specified by the function
%   Screen('Screens'). If no input is given, the monitor with the highest 
%   ID will be used (i.e. the secondary monitor if present).

% History:
% 06/2017 JT adapted from crsLiveTrackDemoShowGazePosition and BubbleDemo

%% Default value for input parameter (if omitted)

% Define default value for the monitor to use 
if nargin<1
    % Get the IDs of all monitors present
    screensAvaiable = Screen('Screens');

   % ID of the stimulus monitor
    WindowID(1) = max(screensAvaiable);
end

%% User input

% Define the images to be displyed in the stimulus screen 
imgFile{1} = 'people-smiling.jpg';
imgFile{2} = 'cambridge-bridge.jpg';

% Define parameters for text
imagesize = [1920 1080];
textString = {...
    'LiveTrack Lightning delivers accurate, precise estimates of eye rotation,',' ',...
    'Direction of Gaze coordinates and pupil size. The equipment is ideal for',' ',...
    'oculomotor research; it is easy and fast to setup and requires minimal',' ',...
    'calibration. Use it for recording saccade metrics, gaze contingent displays,',' ',...
    'and monitoring observer behaviour and pupil response during visual',' ',...
    'psychophysics and cognitive neuroscience experiments.'};
pos = [400 550];
background = [0.5 0.5 0.5];
textcolour = [1 1 1];
fontSize = 25;

% Set the image number to show first 
imgToShow = 3; % set to show the text by default

% Set aperture size in pixels.
aperSize=700;

% Define the raduis of the fixation point (in screen pixels) - used for 
% drift calibration
fixRadIn = 4; % inner circle
fixRadOut = 20;  % outer circle


%% Load and blur images

% Use a simple filter for blurring
sz = 6;     % window size (increase to increase blur)
kernel = ones(sz*2+1,sz*2+1)/(sz*2+1)^2;

% Blur images
clear img imgblur

for i=1:length(imgFile)
    disp(['Blurring image ',imgFile{i},'...']);
    img{i} = imread(imgFile{i});    % read image from file
    imgblur{i}(:,:,1) = conv2(double(img{i}(:,:,1)),kernel); % blur red
    imgblur{i}(:,:,2) = conv2(double(img{i}(:,:,2)),kernel); % blur green 
    imgblur{i}(:,:,3) = conv2(double(img{i}(:,:,3)),kernel); % blue blue
    imgblur{i} = uint8(imgblur{i}); % covnert to 8-bit image
    imgblur{i} = imgblur{i}(1+sz:end-sz,1+sz:end-sz,:);% crop by the kernel window size
end
disp('Done.');

%% Generate text image

% Create a figure and adjust the axis
f = figure('color',background,'units','pixels','position',[0 0 imagesize(1) imagesize(2)],'visible','off');
set(gca,'units','pixels','position',[0 0 imagesize(1)*2 imagesize(2)*2],'visible','off')

% Add the text to the figure
text('units','pixels','position',[pos(1) imagesize(2)-pos(2)],'fontsize',fontSize,'string',textString,'color',textcolour) 

% Convert the figure to an image 
im = getframe(gca); 
close(f);

% Extract the cdata
img{end+1} = im.cdata;
% figure;image(img{end});
% axis off

% Blur text image
disp('Blurring text image...');
i = length(img);
imgblur{i}(:,:,1) = conv2(double(img{i}(:,:,1)),kernel); % blur red
imgblur{i}(:,:,2) = conv2(double(img{i}(:,:,2)),kernel); % blur green 
imgblur{i}(:,:,3) = conv2(double(img{i}(:,:,3)),kernel); % blue blue
imgblur{i} = uint8(imgblur{i}); % covnert to 8-bit image
imgblur{i} = imgblur{i}(1+sz:end-sz,1+sz:end-sz,:);% crop by the kernel window size
disp('Done.');


%% Setup LiveTrack

% Initialise LiveTrack
crsLiveTrackInit;

% Start streaming calibrated results
crsLiveTrackSetResultsTypeCalibrated;

% Start buffering data to the library
crsLiveTrackStartTracking;

% Get the sample rate
[ width, height, sampleRate, offsetX, offsetY, ErrorCode ] = crsLiveTrackGetCaptureConfig; %#ok<ASGLU>

%% Calculate stimulus screen specification

% Use default settings for Psychtoolbox
PsychDefaultSetup(2);

% get the resolution of the stimulus monitor
ResStim=Screen('Resolution', WindowID(1));

% get screen width (in mm)
screenW = Screen('DisplaySize', WindowID(1));

% calculate pixel size (in mm) assuming square pixels
pixSize = screenW/ResStim.width;

%% Open PTB window

% Use openGL
AssertOpenGL;

% Open a full screen window with grey background on stimulus monitor
[winStim, rectStim] = PsychImaging('OpenWindow', WindowID(1), 0.5);

% Set blend function for alpha blending
% Screen('BlendFunction', winStim, GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

% Make image textures
for i=1:length(img)
    % Make the foveated texture
%     texFov(i) = Screen('MakeTexture', winStim, img{i});
    texPer(i) = Screen('MakeTexture', winStim, img{i});

    % Make the periphery texture
%     texPer(i) = Screen('MakeTexture', winStim, imgblur{i});
     texFov(i) = Screen('MakeTexture', winStim, imgblur{i});
end

% Make a texture with the mask
[X,Y]=meshgrid(-aperSize:aperSize, -aperSize:aperSize);
mask(:,:,2)=exp(-((X/(aperSize/3)).^2)-((Y/(aperSize/3)).^2));
masktex=Screen('MakeTexture', winStim, mask);
Screen('FillRect', winStim, [0 0 0 0]); 

% Set default values for parameters used in the screen refresh loop
getOffset = false;
OL=[0 0]; 
OR=[0 0]; 
x=0;
y=0;

% Begin the screen refresh loop (runs once per monitor refresh)
while true
    
   % Get the most recent data sample available, then return it in the
    % structure "Data".
    Data = crsLiveTrackGetLatestEyePosition;
    if Data.tracked && Data.trackedRight
        x = (OL(1)+Data.mmPositions(1)+OR(1)+Data.mmPositionsRight(1))/2;
        y = (OL(2)+Data.mmPositions(2)+OR(2)+Data.mmPositionsRight(2))/2;
    elseif Data.tracked
        x = OL(1)+Data.mmPositions(1);
        y = OL(2)+Data.mmPositions(2);
    elseif Data.trackedRight
        x = OR(1)+Data.mmPositionsRight(1);
        y = OR(2)+Data.mmPositionsRight(2);
    end
    maskx = x/pixSize+rectStim(3)/2;
    masky = y/pixSize+rectStim(4)/2;
   
    % Position of the mask
    rect=[maskx-aperSize masky-aperSize maskx+aperSize+1 masky+aperSize+1];

    % Draw the mask
    Screen('BlendFunction', winStim, GL_ONE, GL_ONE);
    Screen('DrawTexture', winStim, masktex, [], rect);

    % Draw the sharp image
    Screen('BlendFunction', winStim, GL_DST_ALPHA, GL_ZERO);
    Screen('DrawTexture', winStim, texFov(imgToShow))

    % Draw the blurred image
    Screen('BlendFunction', winStim, GL_ONE_MINUS_DST_ALPHA, GL_ONE);
    Screen('DrawTexture', winStim, texPer(imgToShow));            
                        
    % Update the stimulus screen
    Screen('Flip', winStim);
               
    % If the key for launching drift correction is pressed 'getOffset' is
    % true
    if getOffset
        % Call flip to clear the screen 
        Screen('Flip', winStim);

        % Show a white dot at the centre of the screen
        Screen('BlendFunction', winStim, GL_ONE, GL_ZERO);
        Screen('FillOval', winStim, 255, round([ResStim.width/2-fixRadOut...
        ResStim.height/2-fixRadOut ResStim.width/2+fixRadOut ResStim.height/2+fixRadOut]));
        Screen('FillOval', winStim, 0, round([ResStim.width/2-fixRadIn...
        ResStim.height/2-fixRadIn ResStim.width/2+fixRadIn ResStim.height/2+fixRadIn]));

        % Update the stimulus screen
        Screen('Flip', winStim);
        
        % Pause two seconds while showing the dot
        pause(2);
        
        % Get the data from the last second (ignore the first second to
        % allow the subject to move their eyes to the target)
    	Data = crsLiveTrackGetLatestEyePosition(round(sampleRate));
    
        % Get the median gaze position, convert to screen pixels and offset
        % to the centre of the screen, for the left eye
        OL = -median(Data.mmPositions);
        % and for the right eye
        OR = -median(Data.mmPositionsRight);
        
        % Set 'getOffset' to false so we only adjust the offset once
        getOffset = false;
    end

    % Detect if any keys are pressed
    [keyIsDown,~,keyCode]=KbCheck;
    
    % If a key was pressed, take the appropriate action
    if keyIsDown
        % Initiate drift offset if key "1" was pressed
        if keyCode(KbName('1!')) 
            disp('Key 1 pressed for getting offset');
            getOffset = true;
        end
        % Cycle the background image if key "2" was pressed
        if keyCode(KbName('2@')) 
            disp('Key 2 pressed for changing picture');
            if length(img)==imgToShow
                imgToShow=1;
            else
                imgToShow=imgToShow+1;
            end
            % Add a short pause to avoid unintentional repeats
            pause(0.1);
        end
        
        % Exit the script if Esc key is pressed
        if keyCode(KbName('ESCAPE'))
            disp('Esc pressed. Stopping demo');
            break;
        end
    end

end


%% Close LiveTrack and PTB
    
% Stop Tracking
crsLiveTrackStopTracking;

% Clear the buffer
crsLiveTrackClearDataBuffer;

% Close LiveTrack
crsLiveTrackClose;
    
% Close the PTB window and clean up.
sca;
    
