function crsLiveTrackDemoShowGazePosition(WindowID)
%CRSLIVETRACKDEMOSHOWGAZEPOSITION  This demo shows how to get eye tracking
%   data from CRS LiveTrack eye trackers and plot a dot at the current gaze 
%   postion on the screen 
%
%   The demo also shows how to make a quick drift/offset correction by
%   having the particiopant fixate on a single dot on the screen for one
%   second.
%
%   The parameters for the locations of the fixation dots and the fixation
%   criteria can be adjusted in the "User input" section at the beginning
%   of the function.
%
%   The input to this function must be:
%   WindowID - a two-entry vector containing the ID of the control monitor 
%   and the stimulus monitor as specified by the function Screen('Screens').   

% History:
% 09/2016 JT Created
% 05/2017 MAH adapted to use CRS Toolbox for data 
% 06/2017 JT changed to use buffered data from the CRS Toolbox

%% Default values for input parameters (if omitted)

% Define default values for the monitors to use 
if nargin<1
    % Get the IDs of all monitors present
    screensAvaiable = Screen('Screens');

   % ID of experimenter's monitor and stimulus monitor
    WindowID(1) = max(screensAvaiable);
    if length(screensAvaiable)>1
        WindowID(2) = max(screensAvaiable)-1;       
    end
end

%% User input

% Define the images to be displyed in the stimulus screen (images will be
% stretched to fill the full screen 
img{1} = imread('people-smiling.jpg');
img{2} = imread('cambridge-bridge.jpg');

% Set the duration of the sample buffer. The dot shown on the screen is
% placed at the average position of the samples in the buffer. Set buffer
% duration to zero to show the dot at the most recent sample (no averaging)
bufferDuration = 30; % duration in mS

% Mimic window view scale. Sets the size of the mimic window relative to
% the stimilus
cnrViewScale = 0.5; % show a half-sized window of stimulus on control monitor

% Select whether or not to draw gaze point in stimulus window (gaze point
% is always shown in the mimic window) 
drawGazeInStimWin = true;

% Define the raduis of the fixation point (in screen pixels) - used for 
% drift calibration
fixRadIn = 4; % inner circle
fixRadOut = 20;  % outer circle

% Show additional window on experimenter's monitor if more than one monitor
% is configured and if SkipSyncTests is not set (can cause it to hang when
% using DrawText)
if length(WindowID)>1 & ~Screen('Preference', 'SkipSyncTests')
    showMimicwindow=true;
else
    showMimicwindow=false;
end

%% Setup LiveTrack

% Initialise LiveTrack
crsLiveTrackInit;

% Start streaming calibrated results
crsLiveTrackSetResultsTypeCalibrated;

% Start buffering data to the library
crsLiveTrackStartTracking;

% Get the sample rate
[ width, height, sampleRate, offsetX, offsetY, ErrorCode ] = crsLiveTrackGetCaptureConfig; %#ok<ASGLU>

% Calculate how many data samples the fixation duration (fixDur) contains
bufferLength = round((bufferDuration/1000)*sampleRate);

% Make sure the buffer has a length of at least 1
if bufferLength<1
    bufferLength = 1;
end

%% Calculate stimulus screen specification

% Use default settings for Psychtoolbox
PsychDefaultSetup(2);

% get the resolution of the stimulus monitor
ResStim=Screen('Resolution', WindowID(1));

% get screen width (in mm)
screenW = Screen('DisplaySize', WindowID(1));

% calculate pixel size (in mm) assuming square pixels
pixSize = screenW/ResStim.width;

%% Open PTB windows and start collecting data

% Open a full screen window with grey background on stimulus monitor
winStim = PsychImaging('OpenWindow', WindowID(1), 0.5);

% Make the default texture of the background for the stimulus monitor
texStim = Screen('MakeTexture', winStim, img{1});

% Get the resolution of the experimenter's monitor (where the mimic window
% will be shown)
if showMimicwindow
    ResExp = Screen('Resolution', WindowID(2));

    % Open a small screen on the controller monitor to show a mimic of what is
    % displayed on the stimulus screen
    winResExp=[ResStim.width*cnrViewScale ResStim.height*cnrViewScale];
    [winExp, winExpRect] = Screen('OpenWindow', WindowID(2), 0.5,...
        [ResExp.width/2-winResExp(1)/2 ResExp.height/2-winResExp(2)/2 ...
        ResExp.width/2+winResExp(1)/2 ResExp.height/2+winResExp(2)/2]);

    % Set the size of the text
    Screen('TextSize', winExp, 10);

    % Make the default texture of the background for the mimic window
    texExp = Screen('MakeTexture', winExp, img{1});
end

% Set default values for parameters used in the screen refresh loop
getOffset = false;
OL=[0 0]; 
OR=[0 0]; 
tgtLocsL=[0 0];
tgtLocsR=[0 0];
curimg = 1;

% Begin the screen refresh loop (runs once per monitor refresh)
while true
    
    % Get the most recent samples in the buffer 
    Data = crsLiveTrackGetLatestEyePosition(bufferLength);
    
    % Calculate the gaze location in screeen pixel coordinates and apply 
    % offset, for left and right eye
    tgtLocsL(1) = round((OL(1)+mean(Data.mmPositions(:,1)))/pixSize+ResStim.width/2);
    tgtLocsL(2) = round((OL(2)+mean(Data.mmPositions(:,2)))/pixSize+ResStim.height/2);
    tgtLocsR(1) = round((OR(1)+mean(Data.mmPositionsRight(:,1)))/pixSize+ResStim.width/2);
    tgtLocsR(2) = round((OR(2)+mean(Data.mmPositionsRight(:,2)))/pixSize+ResStim.height/2);
    
    % Draw the background image on the stimulus monitor
    Screen('DrawTexture', winStim, texStim, [], [0 0 ResStim.width ResStim.height]);
    
    % Draw a dot for each eye on the stimilus monitor (if enabled by
    % 'drawGazeInStimWin' parameter)
    if drawGazeInStimWin
        if Data.tracked
            Screen('FillOval', winStim, [255 0 0], round([tgtLocsL(1)-fixRadOut...
                    tgtLocsL(2)-fixRadOut tgtLocsL(1)+fixRadOut tgtLocsL(2)+fixRadOut]));
            if showMimicwindow
                DrawFormattedText(winStim,'L',tgtLocsL(1)-fixRadOut/2, tgtLocsL(2)+fixRadOut/2, [1 1 1]);
            end
        end
        if Data.trackedRight
            Screen('FillOval', winStim, [0 255 0], round([tgtLocsR(1)-fixRadOut...
                    tgtLocsR(2)-fixRadOut tgtLocsR(1)+fixRadOut tgtLocsR(2)+fixRadOut]));
            if showMimicwindow
                DrawFormattedText(winStim,'R',tgtLocsR(1)-fixRadOut/2, tgtLocsR(2)+fixRadOut/2, [1 1 1]);
            end
        end
    end
    
    % Update the stimulus screen
    Screen('Flip', winStim);
    
    if showMimicwindow
        % Draw the background image in the mimic window
        Screen('DrawTexture', winExp, texExp, [], winExpRect);

        % Draw a dot for each eye in the mimic monitor 
        if Data.tracked
            Screen('FillOval', winExp, [255 0 0], round([tgtLocsL(1)-fixRadOut...
                    tgtLocsL(2)-fixRadOut tgtLocsL(1)+fixRadOut tgtLocsL(2)+fixRadOut]*cnrViewScale));

            DrawFormattedText(winExp,'L',(tgtLocsL(1)-fixRadOut/2)*cnrViewScale, ...
                (tgtLocsL(2)+fixRadOut/2)*cnrViewScale, [1 1 1]);
        end
        if Data.trackedRight
            Screen('FillOval', winExp, [0 255 0], round([tgtLocsR(1)-fixRadOut...
                    tgtLocsR(2)-fixRadOut tgtLocsR(1)+fixRadOut tgtLocsR(2)+fixRadOut]*cnrViewScale));
            DrawFormattedText(winExp,'R',(tgtLocsR(1)-fixRadOut/2)*cnrViewScale, ...
                (tgtLocsR(2)+fixRadOut/2)*cnrViewScale, [1 1 1]);
        end

        % Draw a text to instruct the experimenter about key functions and 
        % about the coordinates of the current gaze position
        DrawFormattedText(winExp,['Press 1 to adjust offset\nPress 2 to change image\nPress 3 to toggle dots\nLeft X: '...
                 ,num2str(tgtLocsL(1)),' \nLeft Y: ',num2str(tgtLocsL(2)),' \nRight X: '...
                 ,num2str(tgtLocsR(1)),' \nRight Y: ',num2str(tgtLocsR(2))],0, 30, [1 1 1]);

        % Show a red or green dot in the corner of the mimic window indicate
        % the current status of if the eye is tracked or not
        if Data.tracked
           colL = [0 255 0];
        else
           colL = [255 0 0];
        end

        if Data.trackedRight
           colR = [0 255 0];
        else
           colR = [255 0 0];
        end
        Screen('FillOval', winExp, colL,[0 0 20 20]);
        Screen('FillOval', winExp, colR,[30 0 50 20]);

        % Update the mimic window
        Screen('Flip', winExp);
    end
    
    % If the key for launching drift correction is pressed 'getOffset' is
    % true
    if getOffset
        % Call flip to clear the screen 
        Screen('Flip', winStim);
        
        % Show a dot at the centre of the screen with the outer circle at
        % 75% grey and the inner circle black
        Screen('FillOval', winStim, 0.75, round([ResStim.width/2-fixRadOut...
        ResStim.height/2-fixRadOut ResStim.width/2+fixRadOut ResStim.height/2+fixRadOut]));
        Screen('FillOval', winStim, 0, round([ResStim.width/2-fixRadIn...
        ResStim.height/2-fixRadIn ResStim.width/2+fixRadIn ResStim.height/2+fixRadIn]));
        
        % Call flip to show the dot
        Screen('Flip', winStim);
        
        % Pause two seconds while showing the dot
        pause(2);
        
        % Get the data from the last second (ignore the first second to
        % allow the subject to move their eyes to the target)
    	Data = crsLiveTrackGetLatestEyePosition(round(sampleRate));
    
        % Get the median gaze position, convert to screen pixels and offset
        % to the centre of the screen, for the left eye
        OL = -median(Data.mmPositions);
        % and for the right eye
        OR = -median(Data.mmPositionsRight);
        
        % Set 'getOffset' to false so we only adjust the offset once
        getOffset = false;
    end

    % Detect if any keys are pressed
    [keyIsDown,~,keyCode]=KbCheck;
    
    % If a key was pressed, take the appropriate action
    if keyIsDown
        % Initiate drift offset if key "1" was pressed
        if keyCode(KbName('1!')) 
            disp('Key 1 pressed for getting offset');
            getOffset = true;
        end
        % Cycle the background image if key "2" was pressed
        if keyCode(KbName('2@')) 
            disp('Key 2 pressed for changing picture');
            if length(img)==curimg
                curimg=1;
            else
                curimg=curimg+1;
            end
            % Update the texture to be drawn (for both windows)
            if showMimicwindow
                texExp = Screen('MakeTexture', winExp, img{curimg});
                Screen('DrawTexture', winExp, texExp);
            end
            texStim = Screen('MakeTexture', winStim, img{curimg});
            Screen('DrawTexture', winStim, texStim);
            % Add a short pause to avoid unintentional repeats 
            pause(0.1);
        end
        if keyCode(KbName('3#')) 
            disp('Key 3 pressed for toggling gaze dots in stimulus display');
            drawGazeInStimWin = ~drawGazeInStimWin;
            % Add a short pause to avoid unintentional repeats 
            pause(0.1);
        end
        
        % Exit the script if Esc key is pressed
        if keyCode(KbName('ESCAPE'))
            Screen('CloseAll');
            disp('Esc pressed. Stopping demo');
            break;
        end
    end

end


% Stop buffering data to the library
crsLiveTrackStopTracking;

% Clear the buffer
crsLiveTrackClearDataBuffer;

% Close LiveTrack
crsLiveTrackClose;