function  [errDegLeft, errDegRight] = crsLiveTrackDemoGazeCalibrationViSaGe(viewDist, targetsDeg)
% CRSLIVETRACKDEMOGAZECALIBRATIONVISAGE   This demo shows how to calibrate the
% Cambridge Research Systems Livetrack eyetracker with MATLAB and
% ViSaGe
% 
% The demo shows how to aquire raw tracking data (pupil and glint 
% positions) while using ViSaGe to show fixations targets (dots) on a 
% screen. This is done using the function:
% crsLiveTrackGetFixationDataRawViSaGe
% Open and edit this function to change parameters for fixation criteria.
% 
% The raw tracking data is then used, together with information about the
% locations of the targets, to calibrate the gaze measurement for
% LiveTrack. This is done using the API function:
% crsLiveTrackCalibrateDevice
% 
% USAGE:
% crsLiveTrackDemoGazeCalibrationViSaGe(viewDist) sets the viewing distance in
% millimeters (default is 500 mm)
%
% crsLiveTrackDemoGazeCalibration(viewDist, targetsDeg) also sets
% the location of the targets (must be a two-column vector with the first
% column defining the horizontal location of the targets in degrees and the
% seconds column the vertical location).
% 
% History:
% 11/2017   JT Created from crsLiveTrackDemoGazeCalibration

%% Default values for input parameters (if omitted)

% Define the defualt location of the the fixation targets in degrees
% (origin at the centre, x-positive=right, y-positive=down)
if nargin<2
    targetsDeg = [-10 -10;0 -10;10 -10;-10 0;0 0;10 0;-10 10;0 10;10 10];
end

% Define default value for the viewing distance in mm
if nargin<1
    viewDist = 500;
end

%% Routines for the calibration

% Initialise LiveTrack
crsLiveTrackInit;

while true
    % Show fixation dots on the stimulus screen and collect eye tracking data for the calibration 
    [pupilL, glintL, pupilR, glintR, targets] = crsLiveTrackGetFixationDataRawViSaGe(viewDist, targetsDeg);

    % Calculate the calibration matrix parameters for left and right eye
    [trackLeftEye, trackRightEye] = crsLiveTrackGetTracking;

    if trackLeftEye
        errDegLeft = crsLiveTrackCalibrateDevice('left', pupilL, glintL, targets, viewDist);
    else
        errDegLeft = NaN;
    end
    if trackRightEye
        errDegRight = crsLiveTrackCalibrateDevice('right', pupilR, glintR, targets, viewDist);
    else
        errDegRight = NaN;
    end
    
    % Construct a questdlg with two options
    choice = questdlg({'Calibration error:',['Left eye:  ',num2str(errDegLeft),' degrees'],['Right eye: ',num2str(errDegRight),' degrees']}, ...
	'','Accept','Repeat','Accept');
    % Handle response
    switch choice
        case 'Accept'
            disp('Calibration accepted');
            break
        case 'Repeat'
            disp('Repeating calibration');
    end
end    


% Close LiveTrack
crsLiveTrackClose;
