function [pupilL, glintL, pupilR, glintR, targets, order] = crsLiveTrackGetFixationDataRawViSaGe(viewDist, targetsDeg)
%CRSLIVETRACKGETFIXATIONDATARAWVISAGE  This demo shows how raw data from CRS
% LiveTrack eye trackers can be collected for a number of fixations.
%
%   This demo uses ViSaGe functions to drive the display while 
%   acquiring data from CRS LiveTrack devices. The script places dots on
%   the screen at various positions while the participant must fixate the
%   each dot until the a valid data has been aquired. If no valid data as
%   been aquired after a time out period, the fixation point will be
%   skipped and the script moves to the next fixation.
%
%   The function will output the average location of the pupil and glint
%   for each fixation in coordinated of the camera image sensor of the eye
%   tracker. The corresponding location of the fixaxtion dots is returned
%   in pixel coordinates of the screen. An array with the order in which
%   the fixations were taken is also returned.
%
%   The parameters for the locations of the fixaxtion dots and the fixation
%   criteria can be adjusted in the "User input" section at the beginning
%   of the function.
%
%   The input to this function must be:
%   viewDist - perpendicular distance between the participants eyes and the
%   screen in millimetres.


% History:
% 11/2017 JT Created from crsLiveTrackGetFixationDataRaw

%% Default values for input parameters (if omitted)

% Define the defualt location of the the fixation targets in degrees
% (origin at the centre, x-positive=right, y-positive=down)
if nargin<2
    targetsDeg = [-10 -10;0 -10;10 -10;-10 0;0 0;10 0;-10 10;0 10;10 10];
end

% Define default value for the viewing distance in mm
if nargin<1
    viewDist = 500;
end

%% User input

% Time before getting data from fixation (to ignore initial saccade)
setupDelay = 500; % duration in mS

% Define the minimum amount of time required for each fixation (in mS) - 
% decrease to make calibration "easier"/faster
minDur = 1000; % fixation duration in mS

% Time limit for aquiring fixation - if a fixation has not been aquired 
% within this time, the fixation point will be skipped.
fixTimeout = 5; % timeout duration in seconds

% Fixation threshold in camera pixels (all samples taken wihtin the "minDur"
% time must be within this threshold) - increase this to make the  
% calibration "easier" (e.g. for subjects with difficulties making precise
% eye fixations)
fixThreshold = 3.1;

% Define the diameter of the fixation points (in degrees of visual angle)
fixDotOutDeg = 0.6;

%Define the fixation dot colour
FixRGB  = [0 0 0];

% Randomize the order of the targets:
order = randperm(size(targetsDeg,1));
targetsDeg = targetsDeg(order,:);

%% Setup LiveTrack

% Start LiveTrack raw data streaming
crsLiveTrackSetResultsTypeRaw;

% Start buffering data to the library
crsLiveTrackStartTracking;

% Get an estimate of the sample rate
[ width, height, sampleRate, offsetX, offsetY, ErrorCode ] = crsLiveTrackGetCaptureConfig; %#ok<ASGLU>

% Calculate how many data samples the fixation duration (fixDur) contains
fixDurSamples = round((minDur/1000)*sampleRate);

% Find out which eye to get fixation data from
[trackLeftEye, trackRightEye] = crsLiveTrackGetTracking;

%% Calculate stimulus screen specification

% Set up the VSG
global CRS;
vsgInit;
crsSetSpatialUnits(CRS.PIXELUNIT);
  
% Get the resolution of the stimulus monitor
ResStim = crsGetScreenSizePixels;

% Get screen width and height (in mm)
ScreenSize = crsGetScreenSizeMM;
screenW = ScreenSize(1);

% Calculate pixel size (in mm) assuming square pixels
pixSize = screenW/ResStim(1);

% Caliculate millimetres per degree (NB. Only accurate for small eye
% angles)
MMperDeg = tand(1)*viewDist;

% Calculate pixels per degree
pixPerDeg = MMperDeg/pixSize;

% Calculate the radius of the fixation points (in pixels)
fixRadOut = fixDotOutDeg*pixPerDeg/2;

% Target locations in screen pixel coordinates
cnrTarget = [0 0];
tgtLocs(:,1) = round(targetsDeg(:,1)*pixPerDeg+cnrTarget(1)); 
tgtLocs(:,2) = round(targetsDeg(:,2)*pixPerDeg+cnrTarget(2));

%% Open PTB windows and start collecting data

% Open a full screen window with grey background on stimulus monitor
crsSetDrawPage(1);
crsClearPage(1,CRS.BACKGROUND);
crsSetFixationColour(FixRGB);

% Initialise the output data
pupilL = nan(size(targetsDeg,1),2);
pupilR = nan(size(targetsDeg,1),2);
glintL = nan(size(targetsDeg,1),2);
glintR = nan(size(targetsDeg,1),2);
targets = nan(size(targetsDeg,1),2);

% Run a loop over each fixation point and get the fixation data
for i=1:size(targetsDeg,1)
    
    % Draw new fixation point 
    crsClearPage(1,CRS.BACKGROUND);
    crsDrawOval(tgtLocs(i,1:2),[fixRadOut*2,fixRadOut*2]);
    crsPresent;
    
    % This flag will be set to true when a valid fixation has been acquired
    gotFixLeft = false;  
    gotFixRight = false;
       
    tic; % reset fixation timer 
    % Loop until fixation data has been aquired for this dot (or timed out) 
    while 1

        % Make sure there is at lease one sample in the buffer before
        % continuing
        while crsLiveTrackGetResultsCount<1
            if toc>1
                error('ERROR: Could not get any data from LiveTrack!');
            end
        end
        
        % Get the most recent data samples since the buffer was cleared,
        % up to the number specified in fixDurSamples
        Data = crsLiveTrackGetLatestEyePosition(fixDurSamples);

        % Calculate the pupil-to-glint vector for x and y direction and
        % left and right eye
        PGvectorLeftX = Data.pupilPositions(:,1)-Data.glintPositions(:,1);
        PGvectorLeftY = Data.pupilPositions(:,2)-Data.glintPositions(:,2);
        PGvectorRightX = Data.pupilPositions(:,1)-Data.glintPositions(:,1);
        PGvectorRightY = Data.pupilPositionsRight(:,2)-Data.glintPositionsRight(:,2);

        % Calculate the maximum difference in the pupil-to-glint 
        % vectors for the samples in the buffer, for left eye
        pgDistLeft = max([max(PGvectorLeftX)-min(PGvectorLeftX) ...
            max(PGvectorLeftY)-min(PGvectorLeftY)]);
        % and for the right eye
        pgDistRight = max([max(PGvectorRightX)-min(PGvectorRightX) ...
            max(PGvectorRightY)-min(PGvectorRightY)]);

        % Check if the maximum vector difference is within the defined
        % limit for a fixation (fixWindow) and all samples are tracked, and
        % the time to wait for fixations (waitTimeForFix) has passed, for
        % the left eye
        if pgDistLeft<=fixThreshold && all(Data.tracked) && toc>setupDelay/1000
            % Check if there are enough samples in the buffer for the
            % defined duration (fixDurSamples)
            if size(PGvectorLeftX,1)>=fixDurSamples && ~gotFixLeft
                % save the data for this fixation
                pupilL(i,:) = median(Data.pupilPositions);
                glintL(i,:) = median(Data.glintPositions);
                disp(['Fixation #',num2str(i),': Found valid fixation for left eye']);
                gotFixLeft = true; % good fixation aquired
            end
        end
        % and for the right eye
        if pgDistRight<=fixThreshold && all(Data.trackedRight) && toc>setupDelay/1000
            % Check if there are enough samples in the buffer for the
            % defined duration (fixDurSamples)
            if size(PGvectorRightX,1)>=fixDurSamples && ~gotFixRight
                % save the data for this fixation
                pupilR(i,:) = median(Data.pupilPositionsRight);
                glintR(i,:) = median(Data.glintPositionsRight);
                disp(['Fixation #',num2str(i),': Found valid fixation for right eye']);
                gotFixRight = true; % good fixation aquired
            end
        end
          
        if toc>fixTimeout
            if ~gotFixLeft && trackLeftEye
                disp(['Fixation #',num2str(i),': Did not get fixation for left eye (timeout)']);
            end
            if ~gotFixRight && trackRightEye
                disp(['Fixation #',num2str(i),': Did not get fixation for right eye (timeout)']);
            end
            break; % fixation timed out
        end
        
        % Exit if all eyes that are enabled have got a fixation
        if (gotFixLeft || ~trackLeftEye) && (gotFixRight || ~trackRightEye)
            break;
        end
    end
                 
    % Return the target positions in millimetres
    targets(i,:) = targetsDeg(i,:)*MMperDeg;

end

% Remove the last fixation dot
crsClearPage(1,CRS.BACKGROUND);

% Stop buffering data to the library
crsLiveTrackStopTracking;

% Clear the data in the buffer
crsLiveTrackClearDataBuffer;

% put data back in order before returning it
pupilL(order,:) = pupilL;
glintL(order,:) = glintL;
pupilR(order,:) = pupilR;
glintR(order,:) = glintR;
targets(order,:) = targets;


end