% crsLiveTrackDemoPupilSizeCalibration
% CRSLIVETRACKDEMOPUPILSIZECALIBRATION   This demo shows how to calibrate 
% the pupil size scale on Cambridge Research Systems Livetrack eyetrackers
% with MATLAB
%
% The demo shows how to use an artificial pupil (e.g. black dot) of a known
% size to calculate the the pupil size scale factor. The unscaled, raw data
% returns the diameter of the major axis of the ellipse fit measured in
% number of pixels on the camera sensor. The calculated scale factor
% therefore has the units of 'camera pixels to millimetres'.
%
% Note that the script needs the CRS LiveTrack Toolbox for MATLAB.
%
% History:
% 09/2014   JT Created  
% 07/2017   JT Updated to work with CRS LiveTrack Toolbox for MATLAB 
% 09/2017   JT Updated to automatically upload the calibration to LiveTrack

button = questdlg({'Please place the Scale Calibration Stick in front of the camera at',...
    'the same location as the left eye would be and ',...
    'select the diameter of the tracked dot:',...
        },'','3 mm','4 mm','5 mm','4 mm');

if isempty(button)
    waitfor(msgbox('Program execution cancelled.'))
    error('Program stopped');
end
if strcmp(button,'Cancel')
    waitfor(msgbox('Program execution cancelled.'))
    error('Program stopped');
end
if strcmp(button,'3 mm')
       disp('3 mm dot selected');
       dotSize=3;
end
if strcmp(button,'4 mm')
       disp('4 mm dot selected');
       dotSize=4;
end
if strcmp(button,'5 mm')
       disp('5 mm dot selected');
       dotSize=5;
end

% Initialise LiveTrack
crsLiveTrackInit;

% Start LiveTrack raw data streaming
crsLiveTrackSetResultsTypeRaw;

% Clear the data in the buffer
crsLiveTrackClearDataBuffer;

% Start buffering data to the library
crsLiveTrackStartTracking;

% Collect samples for two seconds
pause(2);

% Stop buffering data to the library
crsLiveTrackStopTracking;

% Get all samples
Data = crsLiveTrackGetBufferedEyePositions(true);

% Only use valid samples (pupil size > 0)
pupilDiameter = Data.pupilDiameter(Data.pupilDiameter>0);

% Only show results if more than ten samples were collected 
if length(pupilDiameter)<10
    % Close the connection to LiveTrack
    crsLiveTrackClose;
    error('Too few valid samples in measurement. Try again.')

    %and if the variability of the data is low
elseif std(pupilDiameter)>0.2
    % Close the connection to LiveTrack
    crsLiveTrackClose;
	error('Too much variability in the measurement. Try again.')
else   
    % Upload the pupil size calibration to LiveTrack
    crsLiveTrackSetPupilCalibration(dotSize, median(pupilDiameter));

    disp('Pupil size calibration has now been uploaded to LiveTrack');
    
    % Get the pupil size calibration from LiveTrack
    pupilSizeScaleFactor = crsLiveTrackGetPupilCalibration;
    
    disp(['Pupil scale factor = ',num2str(median(pupilDiameter)/dotSize),' (camera pixels per mm)']);
    
    % Continously stream calibrated pupil size until message box is closed
    crsLiveTrackSetResultsTypeCalibrated;
    
    % Start buffering data to the library
    crsLiveTrackStartTracking;

    % Open a message box
    h = msgbox({'---------------------------------------------------',...
        '---------------------------------------------------'});
    
    while ishandle(h)
        
        % Get the last samples in the buffer
        Data = crsLiveTrackGetLatestEyePosition;
    
        % Update message box with the latest data
        set(findobj(h,'Tag','MessageBox'),'String',...
            {'Pupil scale factor calibrated',['Current pupil size is: ',num2str(Data.pupilDiameter),' mm']});        
        pause(0.1);
    end
    
    % Stop buffering data to the library
    crsLiveTrackStopTracking;
    
    % Close the connection to LiveTrack
    crsLiveTrackClose;
    
end



